/**
 * space_selection_block.js
 * 
 * Provides block templates for displaying Elasticsearch space selection in Slack
 */
const logger = require('../../utils/logger');
const { getAllSpaces } = require('../../services/elastic/elastic_api_service');

const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Generate blocks for space selection for a specific rule
 * 
 * @param {string} ruleId - The ID of the rule to send to a space
 * @param {Object} ruleInfo - Optional rule title and other information
 * @returns {Array} Array of blocks for Slack message
 */
function getSpaceSelectionBlocks(ruleId, ruleInfo = {}) {
  logger.debug(`${FILE_NAME}: Generating space selection blocks for rule: ${ruleId}`);
  
  // Get all configured spaces
  const spaces = getAllSpaces();
  
  if (!spaces || spaces.length === 0) {
    logger.warn(`${FILE_NAME}: No spaces configured for selection blocks`);
    return [{
      type: 'section',
      text: {
        type: 'mrkdwn',
        text: 'No Elasticsearch spaces are configured. Please update your configuration.'
      }
    }];
  }
  
  // Create header block
  const blocks = [
    {
      type: 'header',
      text: {
        type: 'plain_text',
        text: `Select a Space for Rule: ${ruleInfo.title || ruleId}`,
        emoji: true
      }
    },
    {
      type: 'section',
      text: {
        type: 'mrkdwn',
        text: `Please select which Elasticsearch space you want to send this rule to:`
      }
    },
    {
      type: 'divider'
    }
  ];
  
  // Create buttons for each space
  // If we have many spaces, we need to create multiple action blocks
  // Slack only allows 5 buttons per actions block
  const BUTTONS_PER_ACTION = 5;
  let actionBlocks = [];
  let currentActionBlock = {
    type: 'actions',
    elements: []
  };
  
  // Add buttons for each space
  spaces.forEach((space, index) => {
    // Create the button for this space
    const button = {
      type: 'button',
      text: {
        type: 'plain_text',
        text: `${space.emoji || ''} ${space.name}`,
        emoji: true
      },
      value: `send_rule_to_space_${ruleId}_${space.id}`,
      action_id: `send_rule_to_space_${space.id}`
    };
    
    // Add button to current action block
    currentActionBlock.elements.push(button);
    
    // If we've reached the button limit or this is the last space,
    // add the current action block to our collection
    if (currentActionBlock.elements.length === BUTTONS_PER_ACTION || index === spaces.length - 1) {
      actionBlocks.push(currentActionBlock);
      
      // Start a new action block if we have more spaces
      if (index < spaces.length - 1) {
        currentActionBlock = {
          type: 'actions',
          elements: []
        };
      }
    }
  });
  
  // Add all the action blocks to our blocks array
  blocks.push(...actionBlocks);
  
  // Add a cancel button
  blocks.push({
    type: 'actions',
    elements: [
      {
        type: 'button',
        text: {
          type: 'plain_text',
          text: 'Cancel',
          emoji: true
        },
        style: 'danger',
        value: `cancel_space_selection`,
        action_id: 'cancel_space_selection'
      }
    ]
  });
  
  logger.debug(`${FILE_NAME}: Generated ${blocks.length} blocks for space selection`);
  return blocks;
}

module.exports = {
  getSpaceSelectionBlocks
};