/**
 * cli_output_manager.js
 *
 * Centralized manager for CLI output formatting and display
 */
const colors = require('./utils/colors');
const formatters = require('./formatters');

/**
 * CLI Output Manager
 * Handles all CLI output formatting and display
 */
class CliOutputManager {
  /**
   * Create a new CLI Output Manager
   */
  constructor() {
    this.colors = colors;
  }

  /**
   * Display formatted output based on data type
   * 
   * @param {any} data - Data to format and display
   * @param {string} type - Type of data (search_results, details, stats)
   */
  display(data, type) {
    if (!data) {
      console.log('No data returned from the server.');
      return;
    }

    console.log(); // Empty line for spacing

    switch (type) {
      case 'search_results':
        const formattedResults = formatters.formatSigmaSearchResults(data);
        console.log(formatters.renderSigmaSearchResults(formattedResults));
        break;

      case 'details':
        const formattedDetails = formatters.formatSigmaDetails(data);
        console.log(formatters.renderSigmaDetails(formattedDetails));
        break;

      case 'stats':
        const formattedStats = formatters.formatSigmaStats(data);
        console.log(formatters.renderSigmaStats(formattedStats));
        break;

      default:
        // For unknown types, just display JSON
        console.log(JSON.stringify(data, null, 2));
    }
  }

  /**
   * Display a progress message
   * 
   * @param {string} message - Progress message to display
   */
  displayProgress(message) {
    console.log(this.colors.dim(message));
  }

  /**
   * Display an error message
   * 
   * @param {string} message - Error message to display
   */
  displayError(message) {
    console.error(this.colors.error(`Error: ${message}`));
  }

  /**
   * Display a success message
   * 
   * @param {string} message - Success message to display
   */
  displaySuccess(message) {
    console.log(this.colors.success(message));
  }

  /**
   * Display a warning message
   * 
   * @param {string} message - Warning message to display
   */
  displayWarning(message) {
    console.log(this.colors.warning(`Warning: ${message}`));
  }

  /**
   * Display help text
   */
  displayHelp() {
    const helpText = `
Fylgja CLI Help

Basic Sigma Commands:
- search sigma <keyword> - Search for Sigma rules by keyword
- details sigma <rule_id> - Get details about a specific Sigma rule
- stats sigma - Get statistics about Sigma rules database

Advanced Sigma Search Commands:
- search sigma where title contains "ransomware" - Search by title
- search sigma where tags include privilege_escalation - Search by tags
- search sigma where logsource.category == "process_creation" - Search by log source
- search sigma where modified after 2024-01-01 - Search by modification date
- search sigma rules where title contains "ransomware" - Alternative syntax
- search sigma rules where tags include privilege_escalation - Alternative syntax

CLI Commands:
- exit or quit - Exit the CLI
- clear - Clear the terminal screen
- help - Display this help text
`;

    console.log(helpText);
  }
}

module.exports = new CliOutputManager();