/**
 * table_formatter.js
 *
 * Table formatting utilities for CLI output
 */
const colors = require('../utils/colors');
const { wrapText } = require('./text_formatter');

/**
 * Creates a formatted table row with fixed column widths
 * 
 * @param {string[]} columns - Array of column values
 * @param {number[]} widths - Array of column widths
 * @param {Function[]} formatters - Array of formatting functions for each column (optional)
 * @returns {string} Formatted table row
 */
function formatTableRow(columns, widths, formatters = []) {
  return columns.map((value, index) => {
    const formatter = formatters[index] || (text => text);
    return formatter(String(value || '').padEnd(widths[index]));
  }).join('');
}

/**
 * Formats a header row for a table
 * 
 * @param {string[]} headers - Array of header texts
 * @param {number[]} widths - Array of column widths 
 * @returns {string} Formatted header row
 */
function formatTableHeader(headers, widths) {
  return formatTableRow(headers, widths, headers.map(() => colors.header));
}

/**
 * Formats data as a key-value table
 * 
 * @param {Object} data - Object with key-value pairs to display
 * @param {number} keyWidth - Width of the key column
 * @param {number} valueWidth - Width of the value column
 * @param {Function} keyFormatter - Formatting function for keys
 * @returns {string[]} Array of formatted lines
 */
function formatKeyValueTable(data, keyWidth = 25, valueWidth = 50, keyFormatter = colors.key) {
  const lines = [];
  
  for (const [key, value] of Object.entries(data)) {
    if (typeof value !== 'object' || value === null) {
      const formattedKey = keyFormatter(key.padEnd(keyWidth));
      
      // Handle wrapping for multiline values
      const wrappedLines = wrapText(value, valueWidth);
      
      // First line includes the key
      lines.push(`${formattedKey}${wrappedLines[0] || ''}`);
      
      // Additional lines are indented to align with the value column
      for (let i = 1; i < wrappedLines.length; i++) {
        lines.push(`${' '.repeat(keyWidth)}${wrappedLines[i]}`);
      }
    }
  }
  
  return lines;
}

/**
 * Creates a full table with headers and rows
 * 
 * @param {string[]} headers - Table headers
 * @param {Array<Array<string>>} rows - Table data rows
 * @param {number[]} widths - Column widths
 * @returns {string[]} Array of formatted table lines
 */
function formatTable(headers, rows, widths) {
  const lines = [];
  
  // Add header
  lines.push(formatTableHeader(headers, widths));
  
  // Add separator line (optional)
  // lines.push(widths.map(w => '-'.repeat(w)).join(''));
  
  // Add data rows
  rows.forEach(row => {
    lines.push(formatTableRow(row, widths));
  });
  
  return lines;
}

module.exports = {
  formatTableRow,
  formatTableHeader,
  formatKeyValueTable,
  formatTable
};