/**
 * fylgja_command_handler.js
 * 
 * Main handler for the /fylgja slash command
 * Parses natural language commands and routes to appropriate handlers
 *
 * This file is a mess. I've been working mostly on the CLI and other Slash Commands in Slack
 * so development on this command will be put on hold for a bit
 * 
*/

const logger = require('../utils/logger');
const { parseCommand } = require('../lang/command_parser');
const { handleError } = require('../utils/error_handler');
const { handleCommand: handleSigmaSearch, handleComplexSearch } = require('./sigma/sigma_search_entry_handler');
const { handleCommand: handleSigmaDetails } = require('./sigma/sigma_details_handler');
const { handleCommand: handleSigmaStats } = require('./sigma/sigma_stats_handler');
const { handleCommand: handleSigmaCreate } = require('./sigma/sigma_create_handler');
const { handleCommand: handleAlerts } = require('./alerts/alerts_handler');
const { handleCommand: handleCase } = require('./case/case_handler');
const { handleCommand: handleConfig } = require('./config/config_handler');
const { handleCommand: handleStats } = require('./stats/stats_handler');

const FILE_NAME = 'fylgja_command_handler.js';

/**
 * Main handler for the /fylgja command
 * Parses natural language input and routes to appropriate module handlers
 * 
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleCommand = async (command, respond) => {
    try {
        logger.info(`${FILE_NAME}: Received command: ${command.text}`);

        if (!command.text.trim()) {
            logger.warn(`${FILE_NAME}: Empty command received`);
            await respond({
                text: "Please provide a command. Try `/fylgja help` for usage examples.",
                response_type: 'ephemeral'
            });
            return;
        }

        // Parse the natural language command
        const parsedCommand = await parseCommand(command.text);
        logger.debug(`${FILE_NAME}: Parsed command result: ${JSON.stringify(parsedCommand)}`);

        if (!parsedCommand.success) {
            logger.warn(`${FILE_NAME}: Command parsing failed: ${parsedCommand.message}`);
            await respond({
                text: parsedCommand.message || "I couldn't understand that command. Try `/fylgja help` for examples.",
                response_type: 'ephemeral'
            });
            return;
        }

        // Extract the structured command
        const { action, module, params } = parsedCommand.command;
        logger.info(`${FILE_NAME}: Routing command - Module: ${module}, Action: ${action}`);

        // Route to the appropriate handler based on module and action
        switch (module) {
            case 'sigma':
                await handleSigmaCommand(action, params, command, respond);
                break;

            case 'alerts':
                await handleAlerts(command, respond);
                break;

            case 'case':
                await handleCase(command, respond);
                break;

            case 'config':
                await handleConfig(command, respond);
                break;

            case 'stats':
                await handleStats(command, respond);
                break;

            case 'help':
                await handleHelpCommand(respond);
                break;

            default:
                logger.warn(`${FILE_NAME}: Unknown module: ${module}`);
                await respond({
                    text: `Unknown command module: ${module}. Try \`/fylgja help\` for usage examples.`,
                    response_type: 'ephemeral'
                });
        }
    } catch (error) {
        await handleError(error, `${FILE_NAME}: Command handler`, respond, {
            responseType: 'ephemeral'
        });
    }
};

/**
 * Handle Sigma-related commands
 * 
 * @param {string} action - The action to perform
 * @param {Array} params - Command parameters
 * @param {Object} command - The original Slack command
 * @param {Function} respond - Function to send response
 */
const handleSigmaCommand = async (action, params, command, respond) => {
    logger.debug(`${FILE_NAME}: Handling Sigma command - Action: ${action}, Params: ${JSON.stringify(params)}`);

    try {
        switch (action) {
            case 'search':
                // Update the command object with the keyword parameter
                command.text = params[0] || '';
                await handleSigmaSearch(command, respond);
                break;

            case 'complexSearch':
                // Update the command object with the complex query
                command.text = params[0] || '';
                await handleComplexSearch(command, respond);
                break;

            case 'details':
                // Update the command object with the rule ID parameter
                command.text = params[0] || '';
                await handleSigmaDetails(command, respond);
                break;

            case 'stats':
                await handleSigmaStats(command, respond);
                break;

            case 'create':
                // Update the command object with the rule ID parameter
                command.text = params[0] || '';
                await handleSigmaCreate(command, respond);
                break;

            default:
                logger.warn(`${FILE_NAME}: Unknown Sigma action: ${action}`);
                await respond({
                    text: `Unknown Sigma action: ${action}. Try \`/fylgja help\` for usage examples.`,
                    response_type: 'ephemeral'
                });
        }
    } catch (error) {
        await handleError(error, `${FILE_NAME}: Sigma command handler`, respond, {
            responseType: 'ephemeral'
        });
    }
};

/**
 * Handle help command
 * 
 * @param {Function} respond - Function to send response
 */
const handleHelpCommand = async (respond) => {
    try {
        const helpText = `
*Fylgja Command Help*

*Basic Commands:*
• \`/fylgja search <keyword>\` - Search for Sigma rules by keyword
• \`/fylgja details <rule_id>\` - Get details about a specific Sigma rule
• \`/fylgja stats\` - Get statistics about Sigma rules database

*Advanced Search Commands:*
• \`/fylgja search sigma rules where title contains "ransomware"\` - Search by title
• \`/fylgja find rules where tags include privilege_escalation\` - Search by tags
• \`/fylgja search rules where logsource.category == "process_creation"\` - Search by log source
• \`/fylgja find rules where modified after 2024-01-01\` - Search by modification date
• \`/fylgja search where level is "high" and tags include "attack.t1055"\` - Combined search

*Supported Conditions:*
• Title: \`title contains "text"\`
• Description: \`description contains "text"\`
• Log Source: \`logsource.category == "value"\`, \`logsource.product == "value"\`
• Tags: \`tags include "value"\`
• Dates: \`modified after YYYY-MM-DD\`, \`modified before YYYY-MM-DD\`
• Author: \`author is "name"\`
• Level: \`level is "high"\`

*Logical Operators:*
• AND: \`condition1 AND condition2\`
• OR: \`condition1 OR condition2\`

*Pagination:*
• Add \`page=N\` to see page N of results
• Add \`limit=N\` to change number of results per page

For more information, visit the Fylgja documentation.
    `;

        await respond({
            text: helpText,
            response_type: 'ephemeral'
        });
    } catch (error) {
        await handleError(error, `${FILE_NAME}: Help command handler`, respond, {
            responseType: 'ephemeral'
        });
    }
};

module.exports = {
    handleCommand
};