/**
 * handler_registry.js
 * 
 * Centralized registry for command handlers
 * Decouples the CLI from specific handler implementations
 */
const logger = require('../utils/logger');
const FILE_NAME = 'handler_registry.js';

// Create a registry to store handlers
const handlers = {};

/**
 * Registers a handler for a specific module and action
 * @param {string} module - The module name (e.g., 'sigma')
 * @param {string} action - The action name (e.g., 'search')
 * @param {Object} handler - The handler object with a handleCommand method
 */
function registerHandler(module, action, handler) {
  const key = `${module}:${action}`;
  
  if (!handler || typeof handler.handleCommand !== 'function') {
    logger.error(`${FILE_NAME}: Invalid handler for ${key}. Must have handleCommand method.`);
    return;
  }
  
  handlers[key] = handler;
  logger.debug(`${FILE_NAME}: Registered handler for ${key}`);
}

/**
 * Gets a handler for a specific module and action
 * @param {string} module - The module name (e.g., 'sigma')
 * @param {string} action - The action name (e.g., 'search')
 * @returns {Object|null} The handler object or null if not found
 */
function getHandler(module, action) {
  const key = `${module}:${action}`;
  return handlers[key] || null;
}

/**
 * Initializes the handler registry with all available handlers
 */
function initializeRegistry() {
  try {
    // Import all handlers
    const sigmaSearchHandler = require('../handlers/sigma/sigma_search_entry_handler');
    const sigmaDetailsHandler = require('../handlers/sigma/sigma_details_handler');
    const sigmaStatsHandler = require('../handlers/sigma/sigma_stats_handler');
    const sigmaCreateHandler = require('../handlers/sigma/sigma_create_handler');
    
    // Register Sigma handlers
    registerHandler('sigma', 'search', sigmaSearchHandler);
    registerHandler('sigma', 'complexSearch', sigmaSearchHandler);
    registerHandler('sigma', 'details', sigmaDetailsHandler);
    registerHandler('sigma', 'stats', sigmaStatsHandler);
    registerHandler('sigma', 'create', sigmaCreateHandler);
    
    logger.info(`${FILE_NAME}: Handler registry initialized successfully`);
  } catch (error) {
    logger.error(`${FILE_NAME}: Error initializing handler registry: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
  }
}

// Initialize the registry when the module is loaded
initializeRegistry();

module.exports = {
  registerHandler,
  getHandler
};