/**
 * sigma_action_core.js
 * 
 * Core utility functions for Sigma-related Slack actions
 */
const logger = require('../../../utils/logger');
const { handleError } = require('../../../utils/error_handler');
const { getSigmaRuleDetails } = require('../../../services/sigma/sigma_details_service');
const { convertRuleToBackend } = require('../../../services/sigma/sigma_backend_converter');
const { getSigmaRuleDetailsBlocks } = require('../../../blocks/sigma/sigma_details_block');
const { getConversionResultBlocks } = require('../../../blocks/sigma/sigma_conversion_block');

const { SIGMA_CLI_CONFIG } = require('../../../config/appConfig');

const FILE_NAME = 'sigma_action_core.js';

/**
 * Process and display details for a Sigma rule
 * 
 * @param {string} ruleId - The ID of the rule to get details for
 * @param {Function} respond - Function to send response back to Slack
 * @param {boolean} replaceOriginal - Whether to replace the original message
 * @param {string} responseType - Response type (ephemeral or in_channel)
 * @returns {Promise<void>}
 */
const processRuleDetails = async (ruleId, respond, replaceOriginal = false, responseType = 'in_channel') => {
  try {
    if (!ruleId) {
      logger.warn(`${FILE_NAME}: Missing rule ID in processRuleDetails`);
      await respond({
        text: 'Error: Missing rule ID for details',
        replace_original: replaceOriginal,
        response_type: responseType
      });
      return;
    }

    logger.info(`${FILE_NAME}: Processing details for sigma rule: ${ruleId}`);

    // Get Sigma rule details
    logger.info(`${FILE_NAME}: Calling getSigmaRuleDetails with ID: '${ruleId}'`);
    const result = await getSigmaRuleDetails(ruleId);

    if (!result.success) {
      logger.error(`${FILE_NAME}: Rule details retrieval failed: ${result.message}`);
      await respond({
        text: `Error: ${result.message}`,
        replace_original: replaceOriginal,
        response_type: responseType
      });
      return;
    }

    if (!result.explanation) {
      logger.error(`${FILE_NAME}: Rule details succeeded but no explanation object was returned`);
      await respond({
        text: 'Error: Generated details were empty',
        replace_original: replaceOriginal,
        response_type: responseType
      });
      return;
    }

    logger.info(`${FILE_NAME}: Rule ${ruleId} details retrieved successfully`);

    // Generate blocks
    let blocks;
    try {
      blocks = getSigmaRuleDetailsBlocks(result.explanation);
    } catch (blockError) {
      await handleError(blockError, `${FILE_NAME}: Block generation`, respond, {
        replaceOriginal: replaceOriginal,
        responseType: responseType,
        customMessage: `Rule ${result.explanation.id}: ${result.explanation.title}\n${result.explanation.description}`
      });
      return;
    }

    // Respond with the details
    await respond({
      blocks: blocks,
      replace_original: replaceOriginal,
      response_type: responseType
    });
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Process rule details`, respond, {
      replaceOriginal: replaceOriginal,
      responseType: responseType
    });
  }
};

/**
 * Process and convert a Sigma rule to the target backend format
 * 
 * @param {string} ruleId - The ID of the rule to convert
 * @param {Object} config - Configuration for the conversion (backend, target, format)
 * @param {Function} respond - Function to send response back to Slack
 * @param {boolean} replaceOriginal - Whether to replace the original message
 * @param {string} responseType - Response type (ephemeral or in_channel)
 * @returns {Promise<void>}
 */
const processRuleConversion = async (ruleId, config, respond, replaceOriginal = false, responseType = 'in_channel') => {
  try {
    if (!ruleId) {
      logger.warn(`${FILE_NAME}: Missing rule ID in processRuleConversion`);
      await respond({
        text: 'Error: Missing rule ID for conversion',
        replace_original: replaceOriginal,
        response_type: responseType
      });
      return;
    }

    logger.info(`${FILE_NAME}: Processing conversion for sigma rule: ${ruleId}`);

    // Set default configuration from YAML config if not provided
    const conversionConfig = config || {
      backend: SIGMA_CLI_CONFIG.backend,
      target: SIGMA_CLI_CONFIG.target,
      format: SIGMA_CLI_CONFIG.format
    };

    await respond({
      text: `Converting rule ${ruleId} using ${conversionConfig.backend}/${conversionConfig.target} to ${conversionConfig.format}...`,
      replace_original: replaceOriginal,
      response_type: 'ephemeral'
    });

    // Get the rule and convert it
    const conversionResult = await convertRuleToBackend(ruleId, conversionConfig);

    if (!conversionResult.success) {
      logger.error(`${FILE_NAME}: Rule conversion failed: ${conversionResult.message}`);
      await respond({
        text: `Error: ${conversionResult.message}`,
        replace_original: replaceOriginal,
        response_type: responseType
      });
      return;
    }

    // Generate blocks for displaying the result
    let blocks;
    try {
      blocks = getConversionResultBlocks(conversionResult);
    } catch (blockError) {
      await handleError(blockError, `${FILE_NAME}: Block generation`, respond, {
        replaceOriginal: replaceOriginal,
        responseType: responseType,
        customMessage: `Rule ${ruleId} converted successfully. Use the following output with your SIEM:\n\`\`\`\n${conversionResult.output}\n\`\`\``
      });
      return;
    }

    // Respond with the conversion result
    await respond({
      blocks: blocks,
      replace_original: replaceOriginal,
      response_type: responseType
    });
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Process rule conversion`, respond, {
      replaceOriginal: replaceOriginal,
      responseType: responseType
    });
  }
};

module.exports = {
  processRuleDetails,
  processRuleConversion
};