/**
 * sigma_complex_search_handler.js
 * 
 * Handles complex query search commands for Sigma rules
 */
const { searchAndConvertRulesComplex } = require('../../services/sigma/sigma_complex_search_service');
const logger = require('../../utils/logger');
const { handleError } = require('../../utils/error_handler');
const { getSearchResultBlocks } = require('../../blocks/sigma/sigma_search_results_block');
const { getFileName } = require('../../utils/file_utils');
const { extractPaginationParams } = require('../../services/sigma/utils/search_pagination_utils');

const FILE_NAME = getFileName(__filename);

/**
 * Handles the complex sigma-search command
 * @param {Object} command - Slack command object
 * @param {Function} respond - Function to send response
 */
const handleComplexSearch = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing complex search command: ${JSON.stringify(command.text)}`);

    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for complex search`);
      await respond('Invalid command. Usage: /sigma-search where [conditions]');
      return;
    }

    // Extract query string and pagination parameters
    const { text: queryString, page, pageSize } = extractPaginationParams(command.text);

    if (!queryString) {
      logger.warn(`${FILE_NAME}: Missing query in complex search command`);
      await respond('Invalid command: missing query. Usage: /sigma-search where [conditions]');
      return;
    }

    logger.info(`${FILE_NAME}: Performing complex search with query: ${queryString} (page ${page}, size ${pageSize})`);

    await respond({
      text: 'Processing complex search query... This may take a moment.',
      response_type: 'ephemeral'
    });

    // Perform the complex search
    const searchResult = await searchAndConvertRulesComplex(queryString, page, pageSize);
    
    if (!searchResult.success) {
      logger.error(`${FILE_NAME}: Complex search failed: ${searchResult.message}`);
      await respond({
        text: `Search failed: ${searchResult.message}`,
        response_type: 'ephemeral'
      });
      return;
    }

    // Check if we have results
    if (!searchResult.results || searchResult.results.length === 0) {
      logger.warn(`${FILE_NAME}: No rules found matching complex query criteria`);
      await respond({
        text: `No rules found matching the specified criteria.`,
        response_type: 'ephemeral'
      });
      return;
    }

    // Check if this is a CLI request
    const isCliRequest = command.channel_id === 'cli' || command.channel_name === 'cli';

    if (isCliRequest) {
      // For CLI, return just the raw data
      await respond({
        responseData: searchResult.results,
        response_type: 'cli'
      });
      logger.info(`${FILE_NAME}: CLI response sent with ${searchResult.results.length} results`);
      return;
    }

    // For Slack, generate blocks for results
    try {
      // Generate standard search result blocks
      let blocks = getSearchResultBlocks(
        `Complex Query: ${queryString}`,
        searchResult.results,
        searchResult.pagination
      );

      // Customize header for complex search
      if (blocks && blocks.length > 0) {
        blocks[0] = {
          type: "header",
          text: {
            type: "plain_text",
            text: `Sigma Rule Search Results - Query`,
            emoji: true
          }
        };

        // Add query description
        blocks.splice(1, 0, {
          type: "section",
          text: {
            type: "mrkdwn",
            text: `*Query:* \`${queryString}\``
          }
        });
      }

      // Send response
      await respond({
        blocks: blocks,
        response_type: 'ephemeral'
      });

      logger.info(`${FILE_NAME}: Complex search Slack response sent successfully with ${searchResult.results.length} results`);
    } catch (blockError) {
      await handleError(blockError, `${FILE_NAME}: Complex search block generation`, respond, {
        responseType: 'ephemeral',
        customMessage: `Error generating results view: ${blockError.message}`
      });
    }
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Complex search handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

module.exports = {
  handleComplexSearch
};