/**
 * sigma_create_handler.js
 * 
 * Handles Sigma rule conversion requests from Slack commands
 */
const logger = require('../../utils/logger');
const { handleError } = require('../../utils/error_handler');
const { processRuleConversion } = require('./actions/sigma_action_core');
const { SIGMA_CLI_CONFIG } = require('../../config/appConfig');

const FILE_NAME = 'sigma_create_handler.js';

/**
 * Handle the sigma-create command for converting Sigma rules
 * 
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleCommand = async (command, respond) => {
  try {
    logger.debug(`${FILE_NAME}: Processing sigma-create command: ${JSON.stringify(command.text)}`);
    
    if (!command || !command.text) {
      logger.warn(`${FILE_NAME}: Empty command received for sigma-create`);
      await respond('Invalid command. Usage: /sigma-create [id]');
      return;
    }
    
    // Extract rule ID and parameters
    const params = command.text.trim().split(/\s+/);
    const ruleId = params[0];
    
    if (!ruleId) {
      logger.warn(`${FILE_NAME}: Missing rule ID in sigma-create command`);
      await respond('Invalid command: missing rule ID. Usage: /sigma-create [id]');
      return;
    }
    
    await respond({
      text: 'Processing your request... This may take a moment.',
      response_type: 'ephemeral'
    });
    
    // Use configuration from YAML through constants
    const config = {
      backend: SIGMA_CLI_CONFIG.backend,
      target: SIGMA_CLI_CONFIG.target,
      format: SIGMA_CLI_CONFIG.format
    };
    
    await processRuleConversion(ruleId, config, respond, false, 'in_channel');
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Create command handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

module.exports = {
  handleCommand
};