/**
 * sigma_stats_handler.js
 *
 * Handles Sigma rule statistics requests from Slack commands
 * Processes requests for database statistics
 */
const logger = require('../../utils/logger');
const { handleError } = require('../../utils/error_handler');
const { getSigmaStats } = require('../../services/sigma/sigma_stats_service');
const { getStatsBlocks } = require('../../blocks/sigma/sigma_stats_block');

const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Handle the sigma-stats command for Sigma rules
 *
 * @param {Object} command - The Slack command object
 * @param {Function} respond - Function to send response back to Slack
 */
const handleCommand = async (command, respond) => {
  try {
    logger.info(`${FILE_NAME}: Processing sigma-stats command`);
    
    // Determine if request is from CLI by checking channel properties
    const isCliRequest = command.channel_id === 'cli' || command.channel_name === 'cli';
    
    await respond({
      text: 'Gathering Sigma rule statistics... This may take a moment.',
      response_type: 'ephemeral'
    });
    
    // Get statistics from service
    const statsResult = await getSigmaStats();
    
    if (!statsResult.success) {
      logger.error(`${FILE_NAME}: Failed to retrieve statistics: ${statsResult.message}`);
      await respond({
        text: `Error retrieving statistics: ${statsResult.message}`,
        response_type: 'ephemeral'
      });
      return;
    }
    
    // For CLI, only include responseData
    if (isCliRequest) {
      await respond({
        responseData: statsResult.stats,
        response_type: 'cli'
      });
    } 
    // For Slack, only generate Block Kit blocks
    else {
      try {
        const blocks = getStatsBlocks(statsResult.stats);
        await respond({
          blocks: blocks,
          response_type: 'in_channel'
        });
      } catch (blockError) {
        await handleError(blockError, `${FILE_NAME}: Block generation`, respond, {
          responseType: 'ephemeral',
          customMessage: 'Error generating statistics view'
        });
      }
    }
  } catch (error) {
    await handleError(error, `${FILE_NAME}: Stats command handler`, respond, {
      responseType: 'ephemeral'
    });
  }
};

module.exports = {
  handleCommand
};