/**
 * index.js
 *
 * Entry point for the Fylgja application.
 * This module initializes the Slack bot server using configuration from fylgja.yml.
 * It handles startup errors and provides logging
 */
const app = require('./app');
const logger = require('./utils/logger');
const { version } = require('../package.json');
const { SERVER_CONFIG } = require('./config/appConfig');

// Start the app
const PORT = SERVER_CONFIG.port;
const ENV = process.env.NODE_ENV || 'development';
const { getFileName } = require('./utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Gracefully handles server shutdown
 * Logs shutdown information and exits the process
 *
 * @param {string} reason - The reason for the shutdown
 */
function handleShutdown(reason) {
  logger.info(`${FILE_NAME}: Shutting down server (${reason})`);
  process.exit(0);
}

/**
 * Immediately-invoked async function to start the server
 * Allows for proper async/await error handling
 */
(async () => {
  try {
    logger.info(`${FILE_NAME}: Starting Fylgja Slack bot v${version} on port ${PORT} (${ENV} environment)`);
    
    // Set up signal handlers for graceful shutdown
    process.on('SIGTERM', () => handleShutdown('SIGTERM'));
    process.on('SIGINT', () => handleShutdown('SIGINT'));
    
    // Handle uncaught exceptions
    process.on('uncaughtException', (error) => {
      logger.error(`${FILE_NAME}: Uncaught exception: ${error.message}`);
      logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
      process.exit(1);
    });
    
    // Handle unhandled promise rejections
    process.on('unhandledRejection', (reason, promise) => {
      logger.error(`${FILE_NAME}: Unhandled rejection at: ${promise}, reason: ${reason}`);
      // Continue running despite unhandled rejection
    });
    
    await app.start(PORT);
    logger.info(`${FILE_NAME}: ✅ Fylgja Slack bot is running on port ${PORT}`);
    logger.info(`${FILE_NAME}: Server URL: http://localhost:${PORT}`);
  } catch (error) {
    logger.error(`${FILE_NAME}: 💥 Fatal error starting app: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    process.exit(1);
  }
})();