/**
 * query_parser.js
 * 
 * Utility to parse complex search queries for Sigma rules
 * Handles conditions like title contains "X", tags include "Y", etc.
 */

const logger = require('../utils/logger');
const FILE_NAME = 'query_parser.js';

/**
 * Parse a complex query string into structured search parameters
 * Supports conditions like:
 * - title contains "ransomware"
 * - logsource.category == "process_creation"
 * - tags include privilege_escalation
 * - modified after 2024-01-01
 * - author is "John Doe"
 * 
 * Also supports logical operators:
 * - AND, and
 * - OR, or
 * 
 * @param {string} queryString - The complex query string to parse
 * @returns {Object} Structured search parameters
 */
function parseComplexQuery(queryString) {
  try {
    logger.debug(`${FILE_NAME}: Parsing complex query: ${queryString}`);
    
    if (!queryString || typeof queryString !== 'string') {
      logger.warn(`${FILE_NAME}: Invalid query string`);
      return { valid: false, error: 'Invalid query string' };
    }
    
    // Initialize the result object
    const result = {
      valid: true,
      conditions: [],
      operator: 'AND' // Default to AND for multiple conditions
    };
    
    // Check for explicit logical operators
    if (/ AND /i.test(queryString)) {
      result.operator = 'AND';
      // Split by AND and parse each part
      const parts = queryString.split(/ AND /i);
      for (const part of parts) {
        const condition = parseCondition(part.trim());
        if (condition) {
          result.conditions.push(condition);
        }
      }
    } else if (/ OR /i.test(queryString)) {
      result.operator = 'OR';
      // Split by OR and parse each part
      const parts = queryString.split(/ OR /i);
      for (const part of parts) {
        const condition = parseCondition(part.trim());
        if (condition) {
          result.conditions.push(condition);
        }
      }
    } else {
      // Single condition
      const condition = parseCondition(queryString.trim());
      if (condition) {
        result.conditions.push(condition);
      }
    }
    
    // If no valid conditions found, mark as invalid
    if (result.conditions.length === 0) {
      result.valid = false;
      result.error = 'No valid search conditions found';
    }
    
    logger.debug(`${FILE_NAME}: Parsed query result: ${JSON.stringify(result)}`);
    return result;
  } catch (error) {
    logger.error(`${FILE_NAME}: Error parsing complex query: ${error.message}`);
    return { 
      valid: false, 
      error: `Error parsing query: ${error.message}`
    };
  }
}

/**
 * Parse a single condition from the query string
 * 
 * @param {string} conditionStr - The condition string to parse
 * @returns {Object|null} Parsed condition object or null if invalid
 */
function parseCondition(conditionStr) {
  logger.debug(`${FILE_NAME}: Parsing condition: ${conditionStr}`);
  
  // Define regex patterns for different condition types
  const patterns = [
    // title contains "value"
    {
      regex: /^(title|name)\s+(contains|has|like|includes)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'title',
        operator: 'contains',
        value: matches[3].trim()
      })
    },
    // description contains "value"
    {
      regex: /^(description|desc)\s+(contains|has|like|includes)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'description',
        operator: 'contains',
        value: matches[3].trim()
      })
    },
    // logsource.category == "value" or logsource.category = "value"
    {
      regex: /^logsource\.(\w+)\s*(==|=|equals?)\s*"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'logsource',
        subfield: matches[1].toLowerCase(),
        operator: 'equals',
        value: matches[3].trim()
      })
    },
    // tags include "value" or tag contains "value"
    {
      regex: /^tags?\s+(includes?|contains|has)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'tags',
        operator: 'contains',
        value: matches[2].trim()
      })
    },
    // modified after YYYY-MM-DD
    {
      regex: /^(modified|updated|created|date)\s+(after|before|on|since)\s+"?(\d{4}-\d{2}-\d{2})"?$/i,
      handler: (matches) => ({
        field: 'date',
        type: matches[1].toLowerCase(),
        operator: matches[2].toLowerCase(),
        value: matches[3].trim()
      })
    },
    // author is "value" or author = "value"
    {
      regex: /^(author|creator)\s+(is|equals?|==|=)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'author',
        operator: 'equals',
        value: matches[3].trim()
      })
    },
    // level is "value" or level = "value"
    {
      regex: /^(level|severity)\s+(is|equals?|==|=)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'level',
        operator: 'equals',
        value: matches[3].trim()
      })
    },
    // id is "value" or id = "value"
    {
      regex: /^(id|identifier)\s+(is|equals?|==|=)\s+"?([^"]+)"?$/i,
      handler: (matches) => ({
        field: 'id',
        operator: 'equals',
        value: matches[3].trim()
      })
    }
  ];
  
  // Try each pattern
  for (const pattern of patterns) {
    const matches = conditionStr.match(pattern.regex);
    if (matches) {
      return pattern.handler(matches);
    }
  }
  
  // If we get here, no patterns matched
  logger.warn(`${FILE_NAME}: No pattern matched condition: ${conditionStr}`);
  
  // Default to simple keyword search if no specific pattern matches
  return {
    field: 'keyword',
    operator: 'contains',
    value: conditionStr.trim()
  };
}

module.exports = {
  parseComplexQuery
};