/**
 * elastic_api_service.js
 * 
 * Service for interacting with Elasticsearch API endpoints
 */
const axios = require('axios');
const logger = require('../../utils/logger');
const { ELASTICSEARCH_CONFIG } = require('../../config/appConfig');

const FILE_NAME = 'elastic_api_service.js';

/**
 * Get Elasticsearch configuration with credentials
 * 
 * @param {string} spaceId - Optional space ID to get configuration for
 * @returns {Object} Configuration object with URL and credentials
 */
const getElasticConfig = (spaceId = null) => {
  // Default config
  const config = {
    protocol: ELASTICSEARCH_CONFIG.protocol || "http",
    hosts: ELASTICSEARCH_CONFIG.hosts || ["localhost:9200"],
    username: ELASTICSEARCH_CONFIG.username || process.env.ELASTIC_USERNAME,
    password: ELASTICSEARCH_CONFIG.password || process.env.ELASTIC_PASSWORD,
    apiEndpoint: ELASTICSEARCH_CONFIG.apiEndpoint
  };
  
  // If space ID provided, find the specific space
  if (spaceId) {
    const space = ELASTICSEARCH_CONFIG.spaces.find(s => s.id === spaceId);
    if (space) {
      // Apply space-specific configuration overrides if they exist
      return {
        ...config,
        space: space
      };
    }
  }
  
  return config;
};

/**
 * Get all configured Elasticsearch spaces
 * 
 * @returns {Array} List of all configured spaces
 */
const getAllSpaces = () => {
  return ELASTICSEARCH_CONFIG.spaces || [];
};

/**
 * Make a generic request to an Elasticsearch API endpoint
 * 
 * @param {Object} options - Request options
 * @param {string} options.method - HTTP method (get, post, put, delete)
 * @param {string} options.endpoint - API endpoint (appended to base URL)
 * @param {string} options.spaceId - Optional space ID
 * @param {Object} options.data - Request payload
 * @param {Object} options.params - URL parameters
 * @param {Object} options.headers - Additional headers
 * @returns {Promise<Object>} - Response object
 */
const makeElasticRequest = async (options) => {
  try {
    const elasticConfig = getElasticConfig(options.spaceId);
    const baseUrl = elasticConfig.protocol + '://' + elasticConfig.hosts[0];
    
    // Build the full URL - use provided endpoint or default API endpoint
    let url = options.endpoint ? 
      `${baseUrl}${options.endpoint.startsWith('/') ? '' : '/'}${options.endpoint}` : 
      elasticConfig.apiEndpoint;
    
    // Handle space in URL if needed
    if (options.spaceId && options.spaceId !== 'default') {
      // Insert space ID into URL: http://localhost:5601/api/detection_engine/rules
      // becomes http://localhost:5601/s/space-id/api/detection_engine/rules
      if (url.includes('/api/')) {
        const urlParts = url.split('/api/');
        url = `${urlParts[0]}/s/${options.spaceId}/api/${urlParts[1]}`;
      }
    }
    
    logger.debug(`${FILE_NAME}: Making ${options.method} request to: ${url}`);
    
    // Set up default headers
    const headers = {
      'Content-Type': 'application/json',
      'kbn-xsrf': 'true',
      ...(options.headers || {})
    };
    
    // Make the request
    const response = await axios({
      method: options.method || 'get',
      url: url,
      headers: headers,
      auth: {
        username: elasticConfig.username,
        password: elasticConfig.password
      },
      data: options.data || null,
      params: options.params || null
    });
    
    // Return a standardized response
    return {
      success: response.status >= 200 && response.status < 300,
      status: response.status,
      data: response.data,
      space: elasticConfig.space
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in Elasticsearch API request: ${error.message}`);
    
    return {
      success: false,
      message: error.response?.data?.message || error.message,
      status: error.response?.status,
      error: error
    };
  }
};

module.exports = {
  makeElasticRequest,
  getElasticConfig,
  getAllSpaces
};