/**
 * sigma_backend_converter.js
 * 
 * Service for converting Sigma rules to various backend SIEM formats
 * Uses the sigma-cli tool for conversion operations
 */
const fs = require('fs');
const path = require('path');
const os = require('os');
const { execSync } = require('child_process');
const logger = require('../../utils/logger');
const { SIGMA_CLI_PATH, SIGMA_CLI_CONFIG } = require('../../config/appConfig');
const { convertSigmaRule } = require('./sigma_converter_service');
const { getRuleYamlContent } = require('../../sigma_db/queries');

const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Convert a Sigma rule to a specific backend format using the sigma-cli
 * 
 * @param {string} ruleId - The ID of the rule to convert
 * @param {Object} config - Configuration for the conversion
 * @param {string} config.backend - Target backend (default from YAML config)
 * @param {string} config.target - Query target (default from YAML config)
 * @param {string} config.format - Output format (default from YAML config)
 * @returns {Promise<Object>} Conversion result with output or error
 */
async function convertRuleToBackend(ruleId, config = {}) {
  try {
    // Validate configuration and set defaults from YAML config
    const backend = config.backend || SIGMA_CLI_CONFIG.backend;
    const target = config.target || SIGMA_CLI_CONFIG.target;
    const format = config.format || SIGMA_CLI_CONFIG.format;
    
    logger.info(`${FILE_NAME}: Converting rule ${ruleId} using backend: ${backend}, target: ${target}, format: ${format}`);
    
    // Verify sigma-cli path
    if (!fs.existsSync(SIGMA_CLI_PATH)) {
      logger.error(`${FILE_NAME}: Sigma CLI not found at path: ${SIGMA_CLI_PATH}`);
      return {
        success: false,
        message: 'Sigma CLI tool not found'
      };
    }
    
    // Get the rule YAML content
    const yamlResult = await getRuleYamlContent(ruleId);
    if (!yamlResult.success || !yamlResult.content) {
      logger.warn(`${FILE_NAME}: Failed to retrieve YAML for rule ${ruleId}: ${yamlResult.message || 'No content'}`);
      return {
        success: false,
        message: yamlResult.message || 'Failed to retrieve rule content'
      };
    }
    
    // Save the YAML to a temporary file
    const tempDir = os.tmpdir();
    const tempFilePath = path.join(tempDir, `sigma_rule_${ruleId}_${Date.now()}.yml`);
    
    logger.debug(`${FILE_NAME}: Writing rule YAML to temp file: ${tempFilePath}`);
    
    try {
      fs.writeFileSync(tempFilePath, yamlResult.content);
    } catch (fileError) {
      logger.error(`${FILE_NAME}: Error writing temporary file: ${fileError.message}`);
      return {
        success: false,
        message: `Error preparing rule for conversion: ${fileError.message}`
      };
    }
    
    // Build the sigma-cli command
    // Command syntax: sigma convert -t "$backend" -p "$target" -f "$format"
    const command = `"${SIGMA_CLI_PATH}" convert -t ${backend} -p ${target} -f ${format} "${tempFilePath}"`;
    
    // Execute the command
    logger.debug(`${FILE_NAME}: Executing sigma-cli command: ${command}`);
    let result;
    
    try {
      result = execSync(command, { encoding: 'utf8' });
    } catch (execError) {
      logger.error(`${FILE_NAME}: Sigma-cli execution error: ${execError.message}`);
      
      // Clean up temporary file
      try {
        fs.unlinkSync(tempFilePath);
      } catch (cleanupError) {
        logger.warn(`${FILE_NAME}: Error removing temporary file: ${cleanupError.message}`);
      }
      
      return {
        success: false,
        message: `Error during rule conversion: ${execError.message}`
      };
    }
    
    // Clean up temporary file
    try {
      fs.unlinkSync(tempFilePath);
    } catch (cleanupError) {
      logger.warn(`${FILE_NAME}: Error removing temporary file: ${cleanupError.message}`);
    }
    
    // Get rule metadata for context
    const ruleData = await convertSigmaRule(ruleId);
    
    if (!ruleData.success || !ruleData.rule) {
      logger.warn(`${FILE_NAME}: Failed to get metadata for rule ${ruleId}`);
      
      // return the conversion output
      return {
        success: true,
        output: result.trim(),
        rule: {
          id: ruleId,
          title: 'Unknown Rule',
          description: 'Rule metadata could not be retrieved'
        },
        conversionDetails: {
          backend,
          target,
          format
        }
      };
    }
    
    // Return the output with rule metadata
    return {
      success: true,
      output: result.trim(),
      rule: ruleData.rule,
      conversionDetails: {
        backend,
        target,
        format
      }
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error converting rule ${ruleId} to backend: ${error.message}`);
    logger.debug(`${FILE_NAME}: Error stack: ${error.stack}`);
    
    return {
      success: false,
      message: `Error converting rule: ${error.message}`
    };
  }
}

module.exports = {
  convertRuleToBackend
};