/**
 * sigma_complex_search_service.js
 * 
 * Service for complex query searches of Sigma rules
 */
const { searchRulesComplex } = require('../../sigma_db/queries');
const { parseComplexQuery } = require('../../lang/query_parser');
const { convertSigmaRule } = require('./sigma_converter_service');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const { validateQueryString, validatePagination } = require('./utils/search_validation_utils');
const { createPaginationInfo, createEmptyResponse } = require('./utils/search_pagination_utils');

const FILE_NAME = getFileName(__filename);

/**
 * Performs a complex search for Sigma rules
 * @param {string} queryString - Complex query string
 * @param {number} page - Page number (1-based)
 * @param {number} pageSize - Results per page
 * @returns {Promise<Object>} Search results with pagination
 */
async function searchSigmaRulesComplex(queryString, page = 1, pageSize = 10) {
  // Validate query string
  const queryValidation = validateQueryString(queryString);
  if (!queryValidation.isValid) {
    return {
      success: false,
      message: queryValidation.message
    };
  }
  
  // Validate pagination
  const { page: validatedPage, pageSize: validatedPageSize, offset } = 
    validatePagination(page, pageSize);
  
  logger.info(`${FILE_NAME}: Performing complex search with query: "${queryValidation.trimmedQuery}" (page ${validatedPage}, size ${validatedPageSize})`);
  
  try {
    // Parse the complex query
    const parsedQuery = parseComplexQuery(queryValidation.trimmedQuery);
    
    if (!parsedQuery.valid) {
      logger.warn(`${FILE_NAME}: Invalid complex query: ${parsedQuery.error}`);
      return {
        success: false,
        message: `Invalid query: ${parsedQuery.error}`
      };
    }
    
    // Execute complex search
    const searchResult = await searchRulesComplex(parsedQuery, validatedPageSize, offset);
    
    // Extract results and total count
    let allResults = [];
    let totalCount = 0;
    
    if (searchResult) {
      if (Array.isArray(searchResult.results)) {
        allResults = searchResult.results;
        totalCount = searchResult.totalCount || 0;
      }
    }
    
    logger.debug(`${FILE_NAME}: Found ${allResults.length} results of ${totalCount} total`);
    
    // Handle no results case
    if (allResults.length === 0) {
      return createEmptyResponse(
        'No rules found matching the complex query criteria',
        createPaginationInfo(validatedPage, validatedPageSize, totalCount)
      );
    }
    
    // Create pagination info
    const pagination = createPaginationInfo(validatedPage, validatedPageSize, totalCount);
    
    // Return successful results
    return {
      success: true,
      results: allResults,
      count: allResults.length,
      query: parsedQuery,
      pagination
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in complex search: ${error.message}`);
    return {
      success: false,
      message: `Error performing complex search: ${error.message}`
    };
  }
}

/**
 * Performs complex search and converts results to full rule objects
 * @param {string} queryString - Complex query string
 * @param {number} page - Page number (1-based)
 * @param {number} pageSize - Results per page
 * @returns {Promise<Object>} Search results with converted rules
 */
async function searchAndConvertRulesComplex(queryString, page = 1, pageSize = 10) {
  try {
    // Perform complex search
    const searchResult = await searchSigmaRulesComplex(queryString, page, pageSize);
    
    if (!searchResult.success || !searchResult.results || searchResult.results.length === 0) {
      return searchResult;
    }
    
    logger.debug(`${FILE_NAME}: Converting ${searchResult.results.length} complex search results to full rule objects`);
    
    // Convert each result to full rule object
    const convertedResults = [];
    for (const result of searchResult.results) {
      try {
        const conversionResult = await convertSigmaRule(result.id);
        if (conversionResult.success && conversionResult.rule) {
          convertedResults.push(conversionResult.rule);
        } else {
          logger.warn(`${FILE_NAME}: Failed to convert rule ${result.id}`);
        }
      } catch (error) {
        logger.error(`${FILE_NAME}: Error converting rule ${result.id}: ${error.message}`);
      }
    }
    
    // Return results with original pagination info
    return {
      success: true,
      results: convertedResults,
      count: convertedResults.length,
      originalCount: searchResult.results.length,
      query: searchResult.query,
      pagination: searchResult.pagination
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in searchAndConvertRulesComplex: ${error.message}`);
    return {
      success: false,
      message: `Error searching and converting rules: ${error.message}`
    };
  }
}

module.exports = {
  searchSigmaRulesComplex,
  searchAndConvertRulesComplex
};