/**
 * complex-search.js
 * Functions for complex searching of Sigma rules
 */

const { getDbConnection } = require('../sigma_db_connection');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const { checkFtsAvailable } = require('./fts-search');
const { buildComplexSqlQuery, buildComplexFtsQuery } = require('./query-builders');
const FILE_NAME = getFileName(__filename);

/**
 * Search for Sigma rules using complex query conditions
 * Supports filtering by multiple attributes like title, logsource, tags, etc.
 * 
 * @param {Object} parsedQuery - The parsed query object containing conditions and operator
 * @param {number} limit - Maximum number of results to return
 * @param {number} offset - Number of results to skip (for pagination)
 * @returns {Promise<Object>} Object with results array and total count
 */
async function searchRulesComplex(parsedQuery, limit = 10, offset = 0) {
  if (!parsedQuery || !parsedQuery.valid) {
    logger.warn(`${FILE_NAME}: Invalid query object provided`);
    return { results: [], totalCount: 0 };
  }

  logger.info(`${FILE_NAME}: Performing complex search with ${parsedQuery.conditions.length} conditions (limit: ${limit}, offset: ${offset})`);

  let db;
  // Declare this at function scope so it's available in the finally block
  let usingFts = false;

  try {
    db = await getDbConnection();
    logger.debug(`${FILE_NAME}: Database connection established for complex search`);

    // Check if FTS5 is available
    const ftsAvailable = await checkFtsAvailable(db);

    if (ftsAvailable) {
      logger.debug(`${FILE_NAME}: Using FTS5 for complex search`);
      // Set flag that we're using FTS
      usingFts = true;
      // Pass db connection to searchRulesComplexFTS and let that function manage it
      const results = await searchRulesComplexFTS(parsedQuery, limit, offset, db);
      return results;
    }

    logger.debug(`${FILE_NAME}: FTS5 not available, using legacy complex search method`);

    // Build the SQL query based on the conditions
    const { sqlQuery, sqlCountQuery, params } = buildComplexSqlQuery(parsedQuery, limit, offset);

    logger.debug(`${FILE_NAME}: Executing complex search SQL: ${sqlQuery}`);
    logger.debug(`${FILE_NAME}: Query parameters: ${JSON.stringify(params)}`);

    // First get the total count of matching results
    const countResult = await new Promise((resolve, reject) => {
      db.get(sqlCountQuery, params.slice(0, params.length - 2), (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: Complex search count query error: ${err.message}`);
          reject(err);
        } else {
          resolve(row || { count: 0 });
        }
      });
    });

    const totalCount = countResult.count;
    logger.debug(`${FILE_NAME}: Total matching rules for complex query: ${totalCount}`);

    // Now get the actual results with pagination
    const results = await new Promise((resolve, reject) => {
      db.all(sqlQuery, params, (err, rows) => {
        if (err) {
          logger.error(`${FILE_NAME}: Complex search query error: ${err.message}`);
          reject(err);
        } else {
          logger.debug(`${FILE_NAME}: Complex search query returned ${rows ? rows.length : 0} results`);
          resolve(rows || []);
        }
      });
    });

    // Format the results
    const formattedResults = results.map(r => ({
      id: r.rule_id,
      title: r.title || r.rule_id
    }));

    logger.debug(`${FILE_NAME}: Returning ${formattedResults.length} results for complex search`);

    return {
      results: formattedResults,
      totalCount
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in complex search operation: ${error.message}`);
    logger.debug(`${FILE_NAME}: Complex search error stack: ${error.stack}`);
    return { results: [], totalCount: 0 };
  } finally {
    // IMPORTANT: Only close the db connection if we're not using FTS
    // When using FTS, let searchRulesComplexFTS manage the connection
    if (db && !usingFts) {
      try {
        await new Promise((resolve) => db.close(() => resolve()));
        logger.debug(`${FILE_NAME}: Database connection closed after complex search operation`);
      } catch (closeError) {
        logger.error(`${FILE_NAME}: Error closing database after complex search: ${closeError.message}`);
      }
    }
  }
}

/**
 * Search for Sigma rules using complex query conditions with FTS5
 * Uses the FTS5 virtual table for faster text searching
 * 
 * @param {Object} parsedQuery - The parsed query object 
 * @param {number} limit - Maximum number of results to return
 * @param {number} offset - Number of results to skip (for pagination)
 * @param {Object} providedDb - Database connection (optional, will create one if not provided)
 * @returns {Promise<Object>} Object with results array and total count
 */
async function searchRulesComplexFTS(parsedQuery, limit = 10, offset = 0, providedDb = null) {
  if (!parsedQuery || !parsedQuery.valid) {
    logger.warn(`${FILE_NAME}: Invalid query object provided for FTS complex search`);
    return { results: [], totalCount: 0 };
  }

  logger.info(`${FILE_NAME}: Performing complex FTS search with ${parsedQuery.conditions.length} conditions`);

  let db;
  let shouldCloseDb = false;

  try {
    // Use provided db connection or create a new one
    if (providedDb) {
      db = providedDb;
    } else {
      db = await getDbConnection();
      shouldCloseDb = true;
      logger.debug(`${FILE_NAME}: Created new database connection for complex FTS search`);
    }

    // Build FTS query from conditions
    const { ftsQuery, whereClause, params } = buildComplexFtsQuery(parsedQuery);

    logger.debug(`${FILE_NAME}: FTS query: "${ftsQuery}", additional where: ${whereClause ? whereClause : 'none'}`);
    logger.debug(`${FILE_NAME}: Query parameters: ${JSON.stringify(params)}`);

    // Build count query
    let countQuery;
    let countParams;

    if (whereClause) {
      countQuery = `
        SELECT COUNT(*) as count
        FROM rule_search
        WHERE rule_search MATCH ?
        AND ${whereClause}
      `;
      countParams = [ftsQuery, ...params];
    } else {
      countQuery = `
        SELECT COUNT(*) as count
        FROM rule_search
        WHERE rule_search MATCH ?
      `;
      countParams = [ftsQuery];
    }

    // Get total count
    const countResult = await new Promise((resolve, reject) => {
      db.get(countQuery, countParams, (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: Complex FTS count query error: ${err.message}`);
          reject(err);
        } else {
          resolve(row || { count: 0 });
        }
      });
    });

    const totalCount = countResult.count;
    logger.debug(`${FILE_NAME}: Total matching rules for complex FTS query: ${totalCount}`);

    // Build results query with pagination
    let searchQuery;
    let searchParams;

    if (whereClause) {
      searchQuery = `
        SELECT rule_id, title
        FROM rule_search
        WHERE rule_search MATCH ?
        AND ${whereClause}
        ORDER BY rank
        LIMIT ? OFFSET ?
      `;
      searchParams = [ftsQuery, ...params, limit, offset];
    } else {
      searchQuery = `
        SELECT rule_id, title
        FROM rule_search
        WHERE rule_search MATCH ?
        ORDER BY rank
        LIMIT ? OFFSET ?
      `;
      searchParams = [ftsQuery, limit, offset];
    }

    // Get paginated results
    const results = await new Promise((resolve, reject) => {
      db.all(searchQuery, searchParams, (err, rows) => {
        if (err) {
          logger.error(`${FILE_NAME}: Complex FTS search query error: ${err.message}`);
          reject(err);
        } else {
          logger.debug(`${FILE_NAME}: Complex FTS search query returned ${rows ? rows.length : 0} results`);
          resolve(rows || []);
        }
      });
    });

    // Format the results
    const formattedResults = results.map(r => ({
      id: r.rule_id,
      title: r.title || r.rule_id
    }));

    logger.debug(`${FILE_NAME}: Returning ${formattedResults.length} results for complex FTS search`);

    return {
      results: formattedResults,
      totalCount
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in complex FTS search operation: ${error.message}`);
    logger.debug(`${FILE_NAME}: Complex FTS search error stack: ${error.stack}`);
    return { results: [], totalCount: 0 };
  } finally {
    // Only close the database if we created it AND we're not in the middle of a transaction
    if (db && shouldCloseDb) {
      try {
        await db.close();
        logger.debug(`${FILE_NAME}: Database connection closed after complex FTS search`);
      } catch (closeError) {
        logger.error(`${FILE_NAME}: Error closing database after complex FTS search: ${closeError.message}`);
      }
    }
  }
}

module.exports = {
  searchRulesComplex,
  searchRulesComplexFTS
};