/**
 * fts-search.js
 * Functions for Full Text Search (FTS) of Sigma rules
 */

const { getDbConnection } = require('../sigma_db_connection');
const logger = require('../../utils/logger');
const { getFileName } = require('../../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Check if FTS5 virtual table is available
 * 
 * @param {Object} db - Database connection
 * @returns {Promise<boolean>} Whether FTS5 is available
 */
async function checkFtsAvailable(db) {
  try {
    const result = await new Promise((resolve, reject) => {
      db.get("SELECT name FROM sqlite_master WHERE type='table' AND name='rule_search'", (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: Error checking for FTS5 table: ${err.message}`);
          reject(err);
        } else {
          resolve(row !== undefined);
        }
      });
    });

    logger.debug(`${FILE_NAME}: FTS5 table availability check: ${result ? 'Available' : 'Not available'}`);
    return result;
  } catch (error) {
    logger.error(`${FILE_NAME}: Error checking FTS availability: ${error.message}`);
    return false;
  }
}

/**
 * Search for Sigma rules using FTS5
 * Performs a full-text search and returns matching rules with pagination
 * 
 * @param {string} keyword - The keyword to search for
 * @param {number} limit - Maximum number of results to return (default: 10)
 * @param {number} offset - Number of results to skip (for pagination, default: 0)
 * @returns {Promise<Object>} Object with results array and total count
 */
async function searchRulesFTS(keyword, limit = 10, offset = 0) {
  if (!keyword) {
    logger.warn(`${FILE_NAME}: Empty search keyword provided for FTS search`);
    return { results: [], totalCount: 0 };
  }

  // Prepare FTS query - add * for prefix matching if not already present
  let ftsQuery = keyword.trim();
  if (!ftsQuery.endsWith('*')) {
    ftsQuery = `${ftsQuery}*`;
  }

  logger.info(`${FILE_NAME}: Performing FTS search with query: "${ftsQuery}" (limit: ${limit}, offset: ${offset})`);

  let db;
  try {
    db = await getDbConnection();
    logger.debug(`${FILE_NAME}: Database connection established for FTS search`);

    // First get the total count of matching rules
    const countQuery = `
      SELECT COUNT(*) as count
      FROM rule_search
      WHERE rule_search MATCH ?
    `;

    const countResult = await new Promise((resolve, reject) => {
      db.get(countQuery, [ftsQuery], (err, row) => {
        if (err) {
          logger.error(`${FILE_NAME}: FTS count query error: ${err.message}`);
          reject(err);
        } else {
          resolve(row || { count: 0 });
        }
      });
    });

    const totalCount = countResult.count;
    logger.debug(`${FILE_NAME}: Total matching rules for FTS query "${ftsQuery}": ${totalCount}`);

    // Now get the actual results with pagination
    const searchQuery = `
      SELECT rule_id, title
      FROM rule_search
      WHERE rule_search MATCH ?
      ORDER BY rank
      LIMIT ? OFFSET ?
    `;

    const results = await new Promise((resolve, reject) => {
      db.all(searchQuery, [ftsQuery, limit, offset], (err, rows) => {
        if (err) {
          logger.error(`${FILE_NAME}: FTS search query error: ${err.message}`);
          reject(err);
        } else {
          logger.debug(`${FILE_NAME}: FTS search query returned ${rows ? rows.length : 0} results`);
          resolve(rows || []);
        }
      });
    });

    logger.debug(`${FILE_NAME}: FTS search results page for query "${ftsQuery}": ${results.length} matches (page ${Math.floor(offset / limit) + 1})`);

    return {
      results: results.map(r => ({ id: r.rule_id, title: r.title || r.rule_id })),
      totalCount
    };
  } catch (error) {
    logger.error(`${FILE_NAME}: Error in FTS search operation: ${error.message}`);
    logger.debug(`${FILE_NAME}: FTS search error stack: ${error.stack}`);
    return { results: [], totalCount: 0 };
  } finally {
    if (db) {
      try {
        await db.close();
        logger.debug(`${FILE_NAME}: Database connection closed after FTS search operation`);
      } catch (closeError) {
        logger.error(`${FILE_NAME}: Error closing database connection after FTS search: ${closeError.message}`);
      }
    }
  }
}

module.exports = {
  checkFtsAvailable,
  searchRulesFTS
};