/**
 * sigma_db_connection.js
 * 
 * This module manages connections to the SQLite database for Sigma rules.
 * It provides functions for creating and closing database connections with Promise support.
 */
const sqlite3 = require('sqlite3').verbose();
const { DB_PATH } = require('../config/appConfig');
const path = require('path');
const logger = require('../utils/logger');

const { getFileName } = require('../utils/file_utils');
const FILE_NAME = getFileName(__filename);

/**
 * Creates and returns a Promise-based connection to the SQLite database
 * Adds promisified methods to the db object for easier async operations
 * 
 * @returns {Promise<Object>} Promise resolving to the database connection object
 */
function getDbConnection() {
  const absolutePath = path.resolve(DB_PATH);
  logger.debug(`${FILE_NAME}: Attempting to connect to database at path: ${absolutePath}`);
  
  return new Promise((resolve, reject) => {
    const db = new sqlite3.Database(DB_PATH, (err) => {
      if (err) {
        logger.error(`${FILE_NAME}: Database connection error: ${err.message}`);
        reject(err);
      } else {
        logger.debug(`${FILE_NAME}: Successfully connected to database at path: ${DB_PATH}`);
        
        // Add promisified methods to db object
        db.getAsync = function(sql, params) {
          return new Promise((resolve, reject) => {
            this.get(sql, params, function(err, row) {
              if (err) {
                logger.error(`${FILE_NAME}: Error in getAsync: ${err.message}`);
                reject(err);
              }
              else resolve(row);
            });
          });
        };
        
        db.allAsync = function(sql, params) {
          return new Promise((resolve, reject) => {
            this.all(sql, params, function(err, rows) {
              if (err) {
                logger.error(`${FILE_NAME}: Error in allAsync: ${err.message}`);
                reject(err);
              }
              else resolve(rows);
            });
          });
        };
        
        resolve(db);
      }
    });
  });
}

/**
 * Safely closes a database connection
 * 
 * @param {Object} db - The database connection to close
 * @returns {Promise<void>} Promise that resolves when the connection is closed
 */
function closeDbConnection(db) {
  return new Promise((resolve, reject) => {
    if (!db) {
      logger.debug(`${FILE_NAME}: No database connection to close`);
      resolve();
      return;
    }
    
    db.close((err) => {
      if (err) {
        logger.error(`${FILE_NAME}: Error closing database: ${err.message}`);
        reject(err);
      } else {
        logger.debug(`${FILE_NAME}: Database connection closed successfully`);
        resolve();
      }
    });
  });
}

module.exports = {
  getDbConnection,
  closeDbConnection
};